package fr.imag.wifibot.gps;

import java.io.InputStream;
import java.nio.charset.Charset;
import java.util.Enumeration;

import org.osgi.framework.BundleActivator;
import org.osgi.framework.BundleContext;
import org.osgi.framework.ServiceReference;

import fr.imag.wifibot.master.IMasterService;
import gnu.io.CommPortIdentifier;
import gnu.io.SerialPort;
import gnu.io.SerialPortEvent;
import gnu.io.SerialPortEventListener;

public class Activator implements BundleActivator, SerialPortEventListener {
	private BundleContext context;
	
	private static final boolean D = true;
	
	private IMasterService service;
	
	private SerialPort serialPort;
    /** The port we're normally going to use. */
	private static final String PORT_NAMES[] = {
			"/dev/ttyUSB1", // Linux
			};
	
	/** Milliseconds to block while waiting for port open */
	private static final int TIME_OUT = 2000;
	/** Default bits per second for COM port. */
	private static final int DATA_RATE = 4800;
	
	/** Buffered input stream from the port */
	private InputStream input;
	
	private StringBuffer incoming_msg;
	private String message;
	private boolean toSend;
	
	public void start(BundleContext context) throws Exception {
		this.context = context;
		System.out.println("Master dit : \"GPS, à toi de jouer !\"");
		
		ServiceReference reference = context
				.getServiceReference(IMasterService.class.getName());
		service = (IMasterService) context.getService(reference);
		
		CommPortIdentifier portId = null;
		Enumeration portEnum = CommPortIdentifier.getPortIdentifiers();

		if(D) System.out.println("Getting ports...");
		// iterate through, looking for the port
		while (portEnum.hasMoreElements()) {
			CommPortIdentifier currPortId = (CommPortIdentifier) portEnum.nextElement();
			if(D) System.out.println("# Port : " + currPortId.getName());
			
			for (String portName : PORT_NAMES) {
				if (currPortId.getName().equals(portName)) {
					portId = currPortId;
					break;
				}
			}
		}

		if (portId == null) {
			System.out.println("Could not find COM port.");
			return;
		}

		try {
			// open serial port, and use class name for the appName.
			serialPort = (SerialPort) portId.open(this.getClass().getName(),
					TIME_OUT);

			// set port parameters
			int data_rate = DATA_RATE;
			
			serialPort.setSerialPortParams(data_rate,
					SerialPort.DATABITS_8,
					SerialPort.STOPBITS_1,
					SerialPort.PARITY_NONE);

			// open the streams
			input = serialPort.getInputStream();

			// add event listeners
			serialPort.addEventListener(this);
			serialPort.notifyOnDataAvailable(true);

			incoming_msg = new StringBuffer();
		} catch (Exception e) {
			System.err.println(e.toString());
		}
	}
	
	public void stop(BundleContext context) throws Exception {
		if (serialPort != null) {
			serialPort.removeEventListener();
			serialPort.close();
		}
		
		System.out.println("Master dit : \"GPS revient dans ta pokeball !\"");
	}
	

	/**
	 * Handle an event on the serial port. Read the data and print it.
	 */
	public synchronized void serialEvent(SerialPortEvent oEvent) {
		if (oEvent.getEventType() == SerialPortEvent.DATA_AVAILABLE) {
			try {
				int available = input.available();
				byte chunk[] = new byte[available];
				input.read(chunk, 0, available);

				// Displayed results are codepage dependent
				String msg = new String(chunk, Charset.forName("UTF-8"));
				
				// GPS Inforad K0
				// We erase the \n and \r
				msg = msg.replace("\n", "").replace("\r", "");

				// If the message part contains a $, it means the end of a message
				// the beginning of another one.
				if (msg.contains("$")) {
					int dollar_index = msg.indexOf("$");
					
					// We append the end of the previous message.
					incoming_msg.append(msg.substring(0, dollar_index));
					
					String message = incoming_msg.toString();
					
					// If it's a position message, we display it.
					if(message.startsWith("$GPGGA") && ChecksumTest.isValid(message))
						toSend = true;
					
					// We reset the buffer.
					incoming_msg.setLength(0);
					// We retrieve the beginning of the new incoming message.
					msg = msg.substring(dollar_index);
				}
				
				// We append the incoming message part.
				incoming_msg.append(msg);
				
				if(toSend) {
					toSend = false;
					service.incomingMessage(message);
				}
				
			} catch (Exception e) {
				System.err.println(e.toString());
			}
		}
		// Ignore all the other eventTypes, but you should consider the other ones.
	}
}
